<?php

namespace Drupal\sharpspring\Form;

use Drupal\Core\Form\ConfigFormBase;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\Core\Link;
use Drupal\sharpspring\SharpSpringClass;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\sharpspring\SharpSpringSettingsInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines the SharpSpring settings form.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * Module settings.
   *
   * @var \Drupal\sharpspring\SharpSpringSettingsInterface
   */
  protected $moduleSettings;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    SharpSpringSettingsInterface $module_settings
  ) {
    parent::__construct($config_factory);
    $this->moduleSettings = $module_settings;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('sharpspring.settings')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'sharpspring_admin_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'sharpspring.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, Request $request = NULL) {
    $sharpspring_config = $this->config('sharpspring.settings');

    $form['account'] = array(
      '#type' => 'fieldset',
      '#title' => t('Tracking Settings'),
    );

    $form['account']['sharpspring_account'] = array(
      '#type' => 'textfield',
      '#title' => t('Web Property ID'),
      '#default_value' => $sharpspring_config->get('sharpspring_account'),
      '#size' => 15,
      '#maxlength' => 20,
      '#required' => TRUE,
      '#description' => t('This ID is unique to each site you want to track separately, and is in the form of KOI-xxxxxxx.'),
    );

    $form['account']['sharpspring_domain'] = array(
      '#type' => 'textfield',
      '#title' => t('Web Property Domain'),
      '#default_value' => $sharpspring_config->get('sharpspring_domain'),
      '#size' => 60,
      '#maxlength' => 60,
      '#required' => TRUE,
      '#description' => t("This is the SharpSpring sub-domain to which tracking information is sent. It is defined in the tracking code as '_setDomain'. Omit the protocol and path: it should fit the following format: koi-xxxxxx.sharpspring.com"),
    );

    $form['account']['finding_values'] = array(
      '#markup' => t('For more information on how to find and set these values please see <a href="http://help.sharpspring.com/customer/portal/articles/1497453-how-to-insert-sharpspring-tracking-code-how-to-add-additional-sites" target="_blank">http://help.sharpspring.com/customer/portal/articles/1497453-how-to-insert-sharpspring-tracking-code-how-to-add-additional-sites</a>'),
    );

    $api_url = Url::fromUri('https://app.sharpspring.com/settings/pubapi');
    $form['api'] = array(
      '#type' => 'fieldset',
      '#title' => t('API Settings'),
      '#description' => t('For advanced integrations, input your Account ID and Secret Key from the :url', array(':url' => Link::fromTextAndUrl('SharpSpring Public API', $api_url))),
    );

    $form['api']['sharpspring_api_account_id'] = array(
      '#title' => t('Account ID'),
      '#type' => 'textfield',
      '#default_value' => $sharpspring_config->get('sharpspring_api_account_id'),
      '#size' => 40,
      '#maxlength' => 40,
    );

    $form['api']['sharpspring_api_secret_key'] = array(
      '#title' => t('Secret Key'),
      '#type' => 'textfield',
      '#default_value' => $sharpspring_config->get('sharpspring_api_secret_key'),
      '#size' => 40,
      '#maxlength' => 40,
    );

    // Visibility settings.
    $visibility = $this->moduleSettings->getSetting('visibility_pages');
    $pages = $this->moduleSettings->getSetting('pages');
    $form['tracking']['page_display'] = [
      '#type' => 'details',
      '#title' => $this->t('Pages'),
      '#group' => 'tracking_scope',
      '#open' => TRUE,
    ];

    if ($visibility == 2) {
      $form['tracking']['page_display'] = [];
      $form['tracking']['page_display']['visibility_pages'] = [
        '#type' => 'value',
        '#value' => 2,
      ];
      $form['tracking']['page_display']['pages'] = [
        '#type' => 'value',
        '#value' => $pages,
      ];
    }
    else {
      $options = [
        $this->t('Every page except the listed pages'),
        $this->t('The listed pages only'),
      ];
      $description_args = [
        '%blog' => 'blog',
        '%blog-wildcard' => 'blog/*',
        '%front' => '<front>',
      ];
      $description = $this->t("Specify pages by using their paths. Enter one path per line. The '*' character is a wildcard. Example paths are %blog for the blog page and %blog-wildcard for every personal blog. %front is the front page.", $description_args);
      $title = $this->t('Pages');
    
      $form['tracking']['page_display']['visibility_pages'] = [
        '#type' => 'radios',
        '#title' => $this->t('Add tracking code to specific pages'),
        '#options' => $options,
        '#default_value' => $visibility,
      ];
      $form['tracking']['page_display']['pages'] = [
        '#type' => 'textarea',
        '#title' => $title,
        '#title_display' => 'invisible',
        '#default_value' => $pages,
        '#description' => $description,
        '#rows' => 10,
      ];
    }

    // Render the role overview.
    $visibility_roles = $this->moduleSettings->getSetting('roles');
    $form['tracking']['role_display'] = [
      '#type' => 'details',
      '#title' => $this->t('Roles'),
      '#group' => 'tracking_scope',
      '#open' => TRUE,
    ];

    $form['tracking']['role_display']['visibility_roles'] = [
      '#type' => 'radios',
      '#title' => $this->t('Add tracking code for specific roles'),
      '#options' => [
        $this->t('Add to the selected roles only'),
        $this->t('Add to every role except the selected ones'),
      ],
      '#default_value' => $this->moduleSettings->getSetting('visibility_roles'),
    ];
    $role_options = array_map(['\Drupal\Component\Utility\SafeMarkup', 'checkPlain'], user_role_names());
    $form['tracking']['role_display']['roles'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Roles'),
      '#default_value' => !empty($visibility_roles) ? $visibility_roles : [],
      '#options' => $role_options,
      '#description' => $this->t('If none of the roles are selected, all users will be tracked. If a user has any of the roles checked, that user will be tracked (or excluded, depending on the setting above).'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();

    // Replace all type of dashes (n-dash, m-dash, minus) with the normal dashes.
    $values['sharpspring_account'] = str_replace(array('–', '—', '−',), '-', $values['sharpspring_account']);

    $this->config('sharpspring.settings')
      ->set('sharpspring_account', $values['sharpspring_account'])
      ->set('sharpspring_domain', $values['sharpspring_domain'])
      ->set('sharpspring_api_account_id', $values['sharpspring_api_account_id'])
      ->set('sharpspring_api_secret_key', $values['sharpspring_api_secret_key'])
      ->set('visibility_pages', $form_state->getValue('visibility_pages'))
      ->set('pages', $form_state->getValue('pages'))
      ->set('visibility_roles', $form_state->getValue('visibility_roles'))
      ->set('roles', $form_state->getValue('roles'))
      ->save();
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);

    $values = $form_state->getValues();

    // Replace all type of dashes (n-dash, m-dash, minus) with the normal dashes.
    $values['sharpspring_account'] = str_replace(array('–', '—', '−',), '-', $values['sharpspring_account']);

    if (!SharpSpringClass::validate_id($values['sharpspring_account'])) {
      $form_state->setErrorByName('sharpspring_account', t('A valid SharpSpring Web Property ID is case sensitive and formatted like KOI-xxxxxxx.'));
    }

    if (!SharpSpringClass::validate_domain($values['sharpspring_domain'])) {
      $form_state->setErrorByName('sharpspring_domain', t('A valid SharpSpring Domain is formatted like koi-XXXXXX.sharpspring.com. Try removing the leading protocol (e.g. https://) or trailing path. (e.g. /somepath).'));
    }

    // Trim some text values.
    $form_state->setValue('pages', trim($values['pages']));
    $form_state->setValue('roles', array_filter($values['roles']));

    // Verify that every path is prefixed with a slash.
    if ($values['visibility_pages'] != 2) {
      $pages = preg_split('/(\r\n?|\n)/', $values['pages']);
      foreach ($pages as $page) {
        if (strpos($page, '/') !== 0 && $page !== '<front>') {
          $form_state->setErrorByName(
            'pages',
            $this->t('Path "@page" not prefixed with slash.', ['@page' => $page])
          );
          // Drupal forms show one error only.
          break;
        }
      }
    }
  }

}
